@@ -76,11 +76,11 @@ class TestFuse:
         self.reg_output(r'^WARNING: Maximum object sizes less than '
                         '1 MiB will degrade performance\.$', count=1)
 
-    def mount(self, expect_fail=None):
+    def mount(self, expect_fail=None, extra_args=[]):
         cmd = (self.s3ql_cmd_argv('mount.s3ql') +
                ["--fg", '--cachedir', self.cache_dir, '--log', 'none',
                 '--compress', 'zlib', '--quiet', self.storage_url, self.mnt_dir,
-                '--authfile', '/dev/null' ])
+                '--authfile', '/dev/null' ] + extra_args)
         self.mount_process = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                                               universal_newlines=True)
         if self.backend_login is not None:
@@ -113,11 +113,11 @@ class TestFuse:
         assert self.mount_process.poll() == 0
         assert not os.path.ismount(self.mnt_dir)
 
-    def fsck(self):
+    def fsck(self, expect_retcode=0, args=[]):
         proc = subprocess.Popen(self.s3ql_cmd_argv('fsck.s3ql') +
                                 [ '--force', '--quiet', '--log', 'none', '--cachedir',
                                   self.cache_dir, '--authfile', '/dev/null',
-                                  self.storage_url ],
+                                  self.storage_url ] + args,
                                 stdin=subprocess.PIPE, universal_newlines=True)
         if self.backend_login is not None:
             print(self.backend_login, file=proc.stdin)
@@ -125,15 +125,23 @@ class TestFuse:
         if self.passphrase is not None:
             print(self.passphrase, file=proc.stdin)
         proc.stdin.close()
-        assert proc.wait() == 0
+        assert proc.wait() == expect_retcode
 
-    def teardown_method(self, method):
+    def umount_fuse(self):
         with open('/dev/null', 'wb') as devnull:
             if platform.system() == 'Darwin':
                 subprocess.call(['umount', '-l', self.mnt_dir], stderr=devnull)
             else:
                 subprocess.call(['fusermount', '-z', '-u', self.mnt_dir],
                                 stderr=devnull)
+
+    def flush_cache(self):
+        subprocess.check_call(self.s3ql_cmd_argv('s3qlctrl') +
+                              [ '--quiet', 'flushcache', self.mnt_dir ])
+
+
+    def teardown_method(self, method):
+        self.umount_fuse()
         os.rmdir(self.mnt_dir)
 
         # Give mount process a little while to terminate
@@ -326,8 +334,7 @@ class TestFuse:
         fstat = os.stat(filename)
         size = fstat.st_size
 
-        subprocess.check_call(self.s3ql_cmd_argv('s3qlctrl') +
-                              [ '--quiet', 'flushcache', self.mnt_dir ])
+        self.flush_cache()
 
         fd = os.open(filename, os.O_RDWR)
 
